const {
  Email,
  services,
  authsome: authsomeHelper,
  Fragment,
  Collection,
} = require('pubsweet-component-helper-service')

module.exports = models => async (req, res) => {
  const { collectionId, fragmentId, recommendationId } = req.params
  let collection, fragment
  try {
    collection = await models.Collection.find(collectionId)
    if (!collection.fragments.includes(fragmentId))
      return res.status(400).json({
        error: `Collection and fragment do not match.`,
      })

    fragment = await models.Fragment.find(fragmentId)
    const recommendation = fragment.recommendations.find(
      rec => rec.id === recommendationId,
    )
    if (!recommendation)
      return res.status(404).json({ error: 'Recommendation not found.' })

    if (recommendation.userId !== req.user)
      return res.status(403).json({
        error: 'Unauthorized.',
      })

    const authsome = authsomeHelper.getAuthsome(models)
    const authsomeObject =
      recommendation.recommendationType === 'editorRecommendation'
        ? collection
        : fragment
    const target = {
      authsomeObject,
      path: req.route.path,
    }
    const canPatch = await authsome.can(req.user, 'PATCH', target)
    if (!canPatch)
      return res.status(403).json({
        error: 'Unauthorized.',
      })

    const UserModel = models.User
    const user = await UserModel.find(req.user)

    Object.assign(recommendation, req.body)
    recommendation.updatedOn = Date.now()
    if (req.body.submittedOn) {
      const fragmentHelper = new Fragment({ fragment })
      const parsedFragment = await fragmentHelper.getFragmentData({
        handlingEditor: collection.handlingEditor,
      })
      const baseUrl = services.getBaseUrl(req)
      const collectionHelper = new Collection({ collection })
      const authors = await fragmentHelper.getAuthorData({
        UserModel,
      })
      const email = new Email({
        UserModel,
        collection,
        parsedFragment,
        baseUrl,
        authors,
      })
      email.setupHandlingEditorEmail({
        reviewSubmitted: true,
        reviewerName: `${user.firstName} ${user.lastName}`,
      })
      if (!['pendingApproval', 'revisionRequested'].includes(collection.status))
        collectionHelper.updateStatus({ newStatus: 'reviewCompleted' })
    }
    await fragment.save()
    return res.status(200).json(recommendation)
  } catch (e) {
    const notFoundError = await services.handleNotFoundError(e, 'Item')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
