const uuid = require('uuid')
const {
  Email,
  services,
  authsome: authsomeHelper,
  Fragment,
  Collection,
} = require('pubsweet-component-helper-service')

module.exports = models => async (req, res) => {
  const { recommendation, comments, recommendationType } = req.body
  if (!services.checkForUndefinedParams(recommendationType))
    return res.status(400).json({ error: 'Recommendation type is required.' })

  const reqUser = await models.User.find(req.user)
  const { collectionId, fragmentId } = req.params

  let collection, fragment

  try {
    collection = await models.Collection.find(collectionId)
    if (!collection.fragments.includes(fragmentId))
      return res.status(400).json({
        error: `Collection and fragment do not match.`,
      })

    fragment = await models.Fragment.find(fragmentId)
  } catch (e) {
    const notFoundError = await services.handleNotFoundError(e, 'Item')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
  const authsome = authsomeHelper.getAuthsome(models)
  const authsomeObject =
    recommendationType === 'editorRecommendation' ? collection : fragment
  const target = {
    authsomeObject,
    path: req.route.path,
  }
  const canPost = await authsome.can(req.user, 'POST', target)
  if (!canPost)
    return res.status(403).json({
      error: 'Unauthorized.',
    })
  fragment.recommendations = fragment.recommendations || []
  const newRecommendation = {
    id: uuid.v4(),
    userId: reqUser.id,
    createdOn: Date.now(),
    updatedOn: Date.now(),
    recommendationType,
  }

  newRecommendation.recommendation = recommendation || undefined
  newRecommendation.comments = comments || undefined
  const UserModel = models.User
  const collectionHelper = new Collection({ collection })
  const fragmentHelper = new Fragment({ fragment })
  const parsedFragment = await fragmentHelper.getFragmentData({
    handlingEditor: collection.handlingEditor,
  })
  const baseUrl = services.getBaseUrl(req)
  const authors = await fragmentHelper.getAuthorData({ UserModel })
  const email = new Email({
    UserModel,
    collection,
    parsedFragment,
    baseUrl,
    authors,
  })
  const FragmentModel = models.Fragment
  if (reqUser.editorInChief || reqUser.admin) {
    if (recommendation === 'return-to-handling-editor')
      collectionHelper.updateStatus({ newStatus: 'reviewCompleted' })
    else {
      collectionHelper.updateFinalStatusByRecommendation({
        recommendation,
      })
      email.setupAuthorsEmail({
        requestToRevision: false,
        publish: recommendation === 'publish',
        FragmentModel,
      })
      email.setupHandlingEditorEmail({
        publish: recommendation === 'publish',
      })
      email.parsedFragment.recommendations = fragment.recommendations
      email.setupReviewersEmail({
        recommendation,
        isSubmitted: true,
        agree: true,
        FragmentModel,
      })
    }
  } else if (recommendationType === 'editorRecommendation') {
    collectionHelper.updateStatusByRecommendation({ recommendation })
    email.setupReviewersEmail({
      recommendation,
      agree: true,
      FragmentModel,
    })
    email.setupReviewersEmail({ agree: false, FragmentModel: models.Fragment })
    email.setupEiCEmail({
      recommendation,
      comments: newRecommendation.comments,
    })
    if (['minor', 'major'].includes(recommendation)) {
      email.parsedFragment.newComments = newRecommendation.comments
      email.setupAuthorsEmail({
        requestToRevision: true,
      })
    }
  }

  fragment.recommendations.push(newRecommendation)
  await fragment.save()
  return res.status(200).json(newRecommendation)
}
