import React, { Fragment } from 'react'
import { head } from 'lodash'
import { th } from '@pubsweet/ui'
import { connect } from 'react-redux'
import styled from 'styled-components'
import { compose, withHandlers, lifecycle, withProps } from 'recompose'
import { ReviewerBreakdown } from 'pubsweet-components-faraday/src/components/Invitations'
import ReviewersDetailsList from 'pubsweet-components-faraday/src/components/Reviewers/ReviewersDetailsList'
import {
  selectReviewers,
  currentUserIsReviewer,
  getCollectionReviewers,
  selectFetchingReviewers,
} from 'pubsweet-components-faraday/src/redux/reviewers'
import { selectRecommendations } from 'pubsweet-components-faraday/src/redux/recommendations'

import { Tabs, Expandable } from '../molecules'
import { ReviewReportCard, ReviewerReportForm } from './'
import { canSeeReviewersReports } from '../../../component-faraday-selectors'

const getTabSections = (collectionId, reviewers, recommendations = []) => [
  {
    key: 1,
    label: 'Reviewers Details',
    content: (
      <ReviewersDetailsList collectionId={collectionId} reviewers={reviewers} />
    ),
  },
  {
    key: 2,
    label: 'Reviewer Reports',
    content: (
      <Fragment>
        {recommendations.length ? (
          recommendations.map((r, index) => (
            <ReviewReportCard i={index + 1} key={r.id} report={r} />
          ))
        ) : (
          <div>No reports submitted yet.</div>
        )}
      </Fragment>
    ),
  },
]

const ReviewsAndReports = ({
  report,
  project,
  version,
  isReviewer,
  mappedReviewers,
  mappedRecommendations,
  canSeeReviewersReports,
  review = {},
  reviewers = [],
  recommendations = [],
}) => (
  <Fragment>
    {canSeeReviewersReports && (
      <Root>
        <Expandable
          label="Reviewers & Reports"
          rightHTML={
            <ReviewerBreakdown
              collectionId={project.id}
              versionId={version.id}
            />
          }
          startExpanded
        >
          <Tabs
            activeKey={1}
            sections={getTabSections(
              project.id,
              mappedReviewers(),
              mappedRecommendations(),
            )}
          />
        </Expandable>
      </Root>
    )}
    {isReviewer && (
      <Root id="review-report">
        <Expandable label="Your Report" startExpanded>
          {report ? (
            <ReviewReportCard report={report} />
          ) : (
            <ReviewerReportForm
              modalKey={`review-${project.id}`}
              project={project}
              review={review}
              version={version}
            />
          )}
        </Expandable>
      </Root>
    )}
  </Fragment>
)

export default compose(
  connect(
    (state, { project }) => ({
      reviewers: selectReviewers(state),
      recommendations: selectRecommendations(state),
      fetchingReviewers: selectFetchingReviewers(state),
      isReviewer: currentUserIsReviewer(state, project.id),
      canSeeReviewersReports: canSeeReviewersReports(state, project.id),
    }),
    { getCollectionReviewers },
  ),
  withHandlers({
    getReviewers: ({ project, getCollectionReviewers }) => () => {
      getCollectionReviewers(project.id)
    },
    mappedRecommendations: ({ recommendations, reviewers }) => () =>
      recommendations.filter(r => r.submittedOn).map(r => ({
        ...r,
        user: reviewers.find(user => user.userId === r.userId),
      })),
    mappedReviewers: ({ recommendations, reviewers }) => () =>
      reviewers.map(r => ({
        ...r,
        review: recommendations.find(rec => rec.userId === r.userId),
      })),
  }),
  withProps(({ recommendations = [] }) => ({
    review: head(recommendations),
    report: head(recommendations.filter(r => r.submittedOn)),
  })),
  lifecycle({
    componentDidMount() {
      const { getReviewers, canSeeReviewersReports } = this.props
      canSeeReviewersReports && getReviewers()
    },
  }),
)(ReviewsAndReports)

// #region styled-components
const Root = styled.div`
  background-color: ${th('colorBackground')};
  margin-top: calc(${th('subGridUnit')} * 2);
  transition: height 0.3s;
`
// #endregion
