import React from 'react'
import styled from 'styled-components'
import { Button, th } from '@pubsweet/ui'
import { compose, withHandlers } from 'recompose'

import { DashboardItems, DashboardFilters } from './'

const Dashboard = ({
  filters,
  getItems,
  dashboard,
  currentUser,
  filterItems,
  filterValues,
  deleteProject,
  getFilterOptions,
  changeFilterValue,
  createDraftSubmission,
  ...rest
}) => (
  <Root>
    <Header>
      <Heading>Manuscripts</Heading>
      <HeaderButtons>
        <Button
          data-test="new-manuscript"
          onClick={createDraftSubmission}
          primary
        >
          New
        </Button>
      </HeaderButtons>
    </Header>
    <DashboardFilters
      changeFilterValue={changeFilterValue}
      getFilterOptions={getFilterOptions}
    />
    <DashboardItems deleteProject={deleteProject} list={getItems()} />
  </Root>
)

export default compose(
  withHandlers({
    getItems: ({
      filters,
      dashboard,
      currentUser,
      filterItems,
      filterValues = {},
    }) => () =>
      filterItems(dashboard.all).sort((a, b) => {
        if (filterValues.order === 'newest') return a.created - b.created < 0
        return a.created - b.created > 0
      }),
  }),
)(Dashboard)

// #region styles
const Root = styled.div`
  display: flex;
  flex-direction: column;
  margin: auto;
  max-width: 60em;
`

const Header = styled.div`
  display: flex;
  justify-content: space-between;
`

const Heading = styled.div`
  color: ${th('colorPrimary')};
  font-size: 1.6em;
  text-transform: uppercase;
`

const HeaderButtons = styled.div`
  display: flex;
`
// #endregion
